/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.component.common;

import java.util.List;
import java.util.Properties;

import javax.jbi.JBIException;
import javax.jbi.component.InstallationContext;
import javax.management.MBeanServer;
import javax.management.ObjectName;

/**
 * An abstract managed bootstrap. This bootstrap class register an MBean object
 * into the JMX server that will be available only during the bootstrap phase.
 * The developer only needs to overide the abstract method that needs to return
 * an object. This object <b>MUST</b> implement an interface. The methods of
 * this interface willl be exposed via JMX.
 *
 * @author Christophe HAMERLING - eBMWebSourcing
 *
 */
public class ManagedBootstrap extends Bootstrap {

    protected ObjectName mbeanName;

    /*
     * (non-Javadoc)
     *
     * @see javax.jbi.component.Bootstrap#cleanUp()
     */
    public void cleanUp() throws JBIException {
        try {
            doCleanUp();
        } catch (JBIException e) {
            throw e;
        } catch (Exception e) {
            throw new JBIException("Error bootstrap clean up", e);
        }
    }

    /**
     * Do the cleanup
     *
     * @throws Exception
     */
    protected void doCleanUp() throws Exception {
        if (this.mbeanName != null) {
            MBeanServer server = installContext.getContext().getMBeanServer();
            if (server == null) {
                throw new JBIException("MBean server is null !");
            }
            if (server.isRegistered(mbeanName)) {
                server.unregisterMBean(mbeanName);
            }
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.jbi.component.Bootstrap#init(javax.jbi.component.InstallationContext)
     */
    public void init(InstallationContext installContext) throws JBIException {
        try {
            super.init(installContext);
            doInit();
        } catch (JBIException e) {
            throw e;
        } catch (Exception e) {
            throw new JBIException("Error during bootstrap init", e);
        }
    }

    /**
     * Really does the init. Registers the MBean into the JMX server.
     *
     * @throws Exception
     */
    protected void doInit() throws Exception {
        ExtensionMBean mBean = new ExtensionMBean(this);

        this.mbeanName = this.installContext.getContext().getMBeanNames()
            .createCustomComponentMBeanName(
                "bootstrap_" + installContext.getComponentName());

        MBeanServer server = this.installContext.getContext().getMBeanServer();

        if (server == null) {
            throw new JBIException("MBean server is null !");
        }

        if (server.isRegistered(this.mbeanName)) {
            server.unregisterMBean(this.mbeanName);
        }

        server.registerMBean(mBean, this.mbeanName);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.component.common.SimpleBootstrap#doConfig()
     */
    protected void doConfig() {
        Properties properties = propertiesManager
            .getProperties();
        List<String> methods = propertiesManager.getExposedMethodNames();

        // additional configuration for component implementation
        doComponentConfiguration(properties, methods);

        // clear configuration properties and assign new values
        propertiesManager.setProperties(properties);
        propertiesManager.setExposedMethods(methods);
    }

    /**
     * Method to be overriden if the component implementation needs to make
     * changes or add/del attributes/methods that will be exposed via JMX. This
     * method is called before the registration of the MBean.
     *
     * @param componentProperties
     * @param methods
     */
    protected void doComponentConfiguration(
        Properties componentProperties, List<String> methods) {
    }

    /**
     * Returns null by default as this class does not provides JMX extension
     * MBean
     *
     * @see javax.jbi.component.ComponentLifeCycle#getExtensionMBeanName()
     */
    public ObjectName getExtensionMBeanName() {
        return mbeanName;
    }
}
