/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.common.interceptor;

import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.objectweb.petals.component.common.AbstractComponent;
import org.objectweb.petals.component.common.util.MessageExchangeWrapper;
import org.objectweb.petals.jbi.descriptor.ServiceUnitExtensibleElement;

/**
 * An abstract class for JBI message interceptor.
 *
 * @author Christophe HAMERLING (chamerling) - eBM WebSourcing
 *
 */
public abstract class Interceptor {

    /**
     * The JBI component of the interceptor (to get access to all the component
     * context)
     */
    protected AbstractComponent component;

    /**
     * The name of the interceptor (defined in the component descriptor)
     */
    protected String name;

    /**
     * The interceptor attributes (defined in the component descriptor)
     */
    protected Map<String, String> parameters;

    /**
     * Interceptor is always activated ?
     */
    protected boolean active = false;

    /**
     * The component logger
     */
    protected Logger logger;

    /**
     *
     * Creates a new instance of {@link Interceptor}
     *
     */
    public Interceptor() {
    }

    /**
     * Initialize the interceptor from its parameters. The parameters are
     * created from the interceptor parameters defined in the JBI descriptor.
     *
     * The interceptor developer must implement this method if he want to
     * initialize its interceptor from the defined parameters.
     *
     * @param parameters
     *            could not be null. The parameters are loaded by the CDK. If no
     *            parameter has been found, the map is empty but not null.
     */
    public final void init(AbstractComponent component, String name,
            boolean active, Map<String, String> parameters, Logger logger) {
        this.component = component;
        this.parameters = parameters;
        this.logger = logger;
        this.name = name;
        this.active = active;

        try {
            onInit();
        } catch (Exception e) {
            logger.log(Level.INFO, "Problem during interceptor initialization");
        }
    }

    /**
     * Additional init. The developer can do extra interceptor configuration
     * here (create configuration objects from parameters, register MBeans...)
     *
     */
    protected void onInit() throws Exception {
    }

    /**
     * The method call before sending a message
     *
     * @param mex
     * @param consumeProvide
     * @param additionalParameters
     *            the parameters defined in the SU for this interceptor. The
     *            user can use this map to retrieve the parameters defined in
     *            the Consume or Provide extensions.
     */
    public abstract void handleMessageSent(MessageExchangeWrapper mex,
            ServiceUnitExtensibleElement consumeProvide,
            Map<String, String> additionalParameters);

    /**
     * The method called just after accepting a message and before the component
     * processing.
     *
     * @param mex
     * @param consumeProvide
     * @param additionalParameters
     *            the parameters defined in the SU for this interceptor. The
     *            user can use this map to retrieve the parameters defined in
     *            the Consume or Provide extensions.
     */
    public abstract void handleMessageAccept(MessageExchangeWrapper mex,
            ServiceUnitExtensibleElement consumeProvide,
            Map<String, String> additionalParameters);

    /**
     * This method can be overridden for interceptor preprocessing.
     *
     * @param mex
     */
    protected void onIntercept(MessageExchangeWrapper mex) throws Exception {
    }

    /**
     * @return the name
     */
    public final String getName() {
        return this.name;
    }

    /**
     * @return the active
     */
    public boolean isActive() {
        return this.active;
    }

    /**
     * @param active the active to set
     */
    public void setActive(boolean active) {
        this.active = active;
    }
}
