/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.common.interceptor.description;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.jbi.JBIException;

import org.objectweb.petals.component.common.ExtensionsConstants;
import org.objectweb.petals.component.common.PEtALSCDKException;
import org.objectweb.petals.component.common.interceptor.Interceptor;
import org.objectweb.petals.component.common.util.CDKExtensions;
import org.objectweb.petals.component.common.util.XMLUtil;
import org.objectweb.petals.jbi.descriptor.Extensions;
import org.objectweb.petals.jbi.descriptor.JBIDescriptor;
import org.objectweb.petals.jbi.descriptor.JBIDescriptorBuilder;
import org.objectweb.petals.jbi.descriptor.JBIDescriptorException;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

/**
 * @author Christophe HAMERLING (chamerling) - eBM WebSourcing
 *
 */
public class InterceptorBuilder {
    private static InterceptorBuilder INSTANCE = null;

    public static final String DELIM = ",";

    public static final String RECEIVE = "receive";

    public static final String SEND = "send";

    public static final String INOUT = "in-out";

    public static final String INTERCEPTOR_NODE = "interceptor";

    public static final String PARAMETER_NODE = "parameter";

    public static final String ORDER_NODE = "order";

    public static final String CLASS_ATTR = "class";

    public static final String ACTIVE_ATTR = "active";

    public static final String MODE_ATTR = "mode";

    public static final String NAME_ATTR = "name";

    /**
     * Get loader instance
     *
     * @return
     */
    public synchronized static final InterceptorBuilder getInstance() {
        if (INSTANCE == null) {
            INSTANCE = new InterceptorBuilder();
        }
        return INSTANCE;
    }

    /**
     * Creates a new instance of {@link InterceptorBuilder}
     *
     */
    private InterceptorBuilder() {
    }

    /**
     * Load the component interceptors.
     *
     * @param rootPath
     */
    public List<InterceptorConfiguration> loadComponentInterceptors(
            String rootPath) throws PEtALSCDKException {
        File jbiFile = new File(rootPath,
                JBIDescriptorBuilder.JBI_DESCRIPTOR_RESOURCE);
        JBIDescriptor descriptor = null;

        try {
            JBIDescriptorBuilder builder = JBIDescriptorBuilder
                    .getInstance();
            descriptor = builder.buildDescriptor(jbiFile.toURI());
        } catch (JBIDescriptorException e) {
            throw new PEtALSCDKException("Can not load the JBI descriptor file");
        }

        Extensions extensions = descriptor.getComponent().getExtensions();
        CDKExtensions cdkExtensions = new CDKExtensions(extensions);

        return loadInterceptors(cdkExtensions);
    }

    /**
     *
     * @param cdkExtensions
     * @return
     */
    public List<InterceptorConfiguration> loadInterceptors(
            CDKExtensions cdkExtensions) {
        List<InterceptorConfiguration> interceptors = new ArrayList<InterceptorConfiguration>();

        Node interceptorNode = XMLUtil.findChild(cdkExtensions
                .getDocumentFragment(), ExtensionsConstants.INTERCEPTORS, true);
        if (interceptorNode != null) {
            List<Node> children = XMLUtil.getNodeChildren(interceptorNode);
            for (Node childNode : children) {
                InterceptorConfiguration conf = loadInterceptorDescriptor(childNode);
                if (conf != null) {
                    interceptors.add(conf);
                }
            }
        }
        return interceptors;
    }

    /**
     * Load the interceptor configuration from the extensions
     *
     * @param extensions
     * @return interceptor configuration defined in the SU or null if no
     *         interceptor extension has been defined in the extensions.
     */
    public ExtensibleElementInterceptorConfiguration loadSUInterceptor(
            CDKExtensions extensions) {
        ExtensibleElementInterceptorConfiguration result = new ExtensibleElementInterceptorConfiguration();

        if (extensions != null && extensions.getDocumentFragment() != null) {
            Node interceptorsNode = XMLUtil.findChild(extensions
                    .getDocumentFragment(), ExtensionsConstants.INTERCEPTORS,
                    true);

            if (interceptorsNode != null) {

                Node inNode = XMLUtil
                        .findChild(interceptorsNode, RECEIVE, true);
                Node outNode = XMLUtil.findChild(interceptorsNode, SEND, true);

                result.setInputInterceptors(getOrderedInterceptors(inNode));
                result.setOutputInterceptors(getOrderedInterceptors(outNode));
            }
        }
        return result;
    }

    /**
     *
     * @param orderNode
     * @return
     */
    private Map<String, InterceptorConfiguration> getOrderedInterceptors(
            Node orderNode) {
        Map<String, InterceptorConfiguration> result = new LinkedHashMap<String, InterceptorConfiguration>();
        if (orderNode != null
                && (orderNode.getNodeName().equals(RECEIVE) || orderNode
                        .getNodeName().equals(SEND))) {

            List<Node> nodes = XMLUtil.getNodeChildren(orderNode);
            for (Node node : nodes) {
                InterceptorConfiguration config = loadInterceptorDescriptor(node);
                result.put(config.getName(), config);
            }
        }
        return result;
    }

    /**
     * Load the interceptor descriptor from the given node.
     *
     * @param interceptorNode
     *            the {@link Node} to create interceptor description from.
     * @return an {@link InterceptorConfiguration} or null if it is not an
     *         interceptor node or if the node is null.
     */
    public InterceptorConfiguration loadInterceptorDescriptor(
            final Node interceptorNode) {
        InterceptorConfiguration result = null;

        if (interceptorNode != null
                && interceptorNode.getNodeName().equals(INTERCEPTOR_NODE)) {
            result = new InterceptorConfiguration();
            Map<String, String> map = new HashMap<String, String>();
            NamedNodeMap atts = interceptorNode.getAttributes();
            if (atts == null) {
                return result;
            }

            for (int i = 0; i < atts.getLength(); i++) {
                Node tmp = atts.item(i);
                map.put(tmp.getNodeName(), tmp.getNodeValue());
            }

            result.setClazz(map.get(CLASS_ATTR));
            result.setName(map.get(NAME_ATTR));
            String active = map.get(ACTIVE_ATTR);
            if (active != null && active.equals("true")) {
                result.setActive(true);
            }

            result.setParameters(map);
        }
        return result;
    }

    /**
     * Load the accept interceptor
     *
     * @return
     */
    public Interceptor loadInstance(String className) throws PEtALSCDKException {
        Interceptor interceptor = null;
        Class clazz = loadClass(className);

        try {
            Object o = clazz.newInstance();
            if (o instanceof Interceptor) {
                interceptor = (Interceptor) clazz.newInstance();
            } else {
                throw new PEtALSCDKException("Invalid class, " + className
                        + " is not an interceptor");
            }

        } catch (InstantiationException e) {
            throw new PEtALSCDKException("Can not instanciate interceptor", e);
        } catch (IllegalAccessException e) {
            throw new PEtALSCDKException("Illegal access to interceptor", e);
        }

        return interceptor;
    }

    /**
     *
     * @param classloader
     * @param className
     * @return
     * @throws JBIException
     */
    protected Class loadClass(String className) throws PEtALSCDKException {
        Class clazz;
        ClassLoader classloader = this.getClass().getClassLoader();
        try {
            clazz = classloader.loadClass(className);
            if (clazz == null) {
                throw new PEtALSCDKException(
                        "Could not find interceptor class : " + className);
            }

        } catch (ClassNotFoundException e) {
            throw new PEtALSCDKException("Class not found interceptor class : "
                    + className, e);
        } catch (NullPointerException e) {
            throw new PEtALSCDKException("Null class : " + className);
        }
        return clazz;
    }
}
