/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * AbstractExternalListener.java - Initial Developper : ofabre
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.common.listener;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;

import org.objectweb.petals.component.common.PEtALSCDKException;
import org.objectweb.petals.component.common.util.CDKExtensions;
import org.objectweb.petals.component.common.util.MessageExchangeWrapper;

/**
 *
 * @author ofabre - EBM Websourcing
 *
 */
public abstract class AbstractExternalListener extends AbstractListener {

    /**
     * Starts the listener. This method is called when the SU that causes this
     * listener instantiation is started.
     *
     * @throws PEtALSCDKException
     *             if an error occurs during external listener starting process
     */
    protected abstract void start() throws PEtALSCDKException;

    /**
     * Stops the listener. This method is called when the SU that causes this
     * listener instantiation is stopped.
     *
     * @throws PEtALSCDKException
     *             if an error occurs during external listener stopping process
     */
    protected abstract void stop() throws PEtALSCDKException;

    /**
     * Send a message, in a synchronous way, to the endpoint specified in the
     * MessageExchange or in the consumes attributes if no information
     * explicitly specified in the MessageExchange (no interface name, service
     * name and endpoint). For exemple, this method can be used by a listener,
     * that listen to messages from external services, in order to send these
     * messages to internal JBI Components. <br/> Automaticaly map
     * "routing.strategy", "transport.compress", "messaging.noack" and
     * "transport.qos" properties, found in SU extensions, to MessageExchange
     * properties. <br/> Process MessageExchange interceptors before sending
     * message. <br/> If no operation is speficied in MessageEchange try to find
     * it in Extensions
     *
     * @param exchange
     *            message exchange to send; must be non-null
     * @param timeoutMS
     *            timeout period, in milliseconds (0 means no timeout); must be
     *            greater than or equal to zero
     * @return true if the message has been sent (if a timeout occurs, return
     *         false)
     * @throws MessagingException
     *             if the address has no endpoint associated to it, or if an
     *             error occurs while sending the message through the container
     */
    public boolean sendSync(final MessageExchangeWrapper exchange, long timeoutMS)
            throws MessagingException {

        if (exchange == null) {
            throw new NullPointerException(
                    "You must provide a non null MessageExchange");
        }

        if (exchange.getEndpoint() == null
                && exchange.getInterfaceName() == null
                && exchange.getService() == null) {
            fillExchangeDestination(exchange);
        }

        return super.sendSync(exchange, timeoutMS);

    }

    /**
     * Send a message, in an asynchronous way, to the endpoint specified in the
     * MessageExchange or in the consumes attributes if no information
     * explicitly specified in the MessageExchange (no interface name, service
     * name and endpoint). For exemple, this method can be used by a listener,
     * that listen to messages from external services, in order to send these
     * messages to internal JBI Components. <br/> Automaticaly map
     * "routing.strategy", "transport.compress", "messaging.noack" and
     * "transport.qos" properties, found in SU extensions, to MessageExchange
     * properties. <br/> Process MessageExchange interceptors before sending
     * message. <br/> If no operation is speficied in MessageEchange try to find
     * it in Extensions. <br/> Add a synchronization mecanism that enable usage
     * of a blocking accept(messageId) method that will be unlock when the
     * response message with the same id reaches the component
     *
     * @see AbstractComponent.accept(String id)
     *
     * @param exchange
     *            message exchange to send; must be non-null
     * @throws MessagingException
     *             if the address has no endpoint associated to it, or if an
     *             error occurs while sending the message through the container
     */
    public void sendAsync(final MessageExchangeWrapper exchange)
            throws MessagingException {

        if (exchange == null) {
            throw new NullPointerException(
                    "You must provide a non null MessageExchange");
        }

        if (exchange.getEndpoint() == null
                && exchange.getInterfaceName() == null
                && exchange.getService() == null) {
            fillExchangeDestination(exchange);
        }

        super.sendAsync(exchange);
    }

    /**
     * Send a message, in an asynchronous way, to the endpoint specified in the
     * MessageExchange or in the consumes attributes if no information
     * explicitly specified in the MessageExchange (no interface name, service
     * name and endpoint). For exemple, this method can be used by a listener,
     * that listen to messages from external services, in order to send these
     * messages to internal JBI Components. <br/> Automaticaly map
     * "routing.strategy", "transport.compress", "messaging.noack" and
     * "transport.qos" properties, found in SU extensions, to MessageExchange
     * properties. <br/> Process MessageExchange interceptors before sending
     * message. <br/> If no operation is speficied in MessageEchange try to find
     * it in Extensions. <br/> No synchronization mecanism activated
     *
     * @param exchange
     *            message exchange to send; must be non-null
     * @throws MessagingException
     *             if the address has no endpoint associated to it, or if an
     *             error occurs while sending the message through the container
     */
    public void send(final MessageExchangeWrapper exchange) throws MessagingException {

        if (exchange == null) {
            throw new NullPointerException(
                    "You must provide a non null MessageExchange");
        }

        if (exchange.getEndpoint() == null
                && exchange.getInterfaceName() == null
                && exchange.getService() == null) {
            fillExchangeDestination(exchange);
        }

        super.send(exchange);
    }

    /**
     * Try to find destination informations in consumes node
     *
     * @param exchange
     */
    private void fillExchangeDestination(final MessageExchange exchange) {
        // Retrieve possible target endpoint
        ServiceEndpoint ep = null;
        if (consumes.getEndpointName() != null
                && consumes.getServiceName() != null) {
            ep = getComponentContext().getEndpoint(consumes.getServiceName(),
                    consumes.getEndpointName());
        }
        // set exchange information
        exchange.setInterfaceName(consumes.getInterfaceName());
        exchange.setService(consumes.getServiceName());
        exchange.setEndpoint(ep);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.component.common.listener.AbstractListener#getCDKExtensions()
     */
    protected final CDKExtensions getCDKExtensions() {
        CDKExtensions extensions = null;
        if (consumes != null) {
            extensions = new CDKExtensions(consumes.getExtensions());
        }
        return extensions;
    }
}
