/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * ExternalListenerManager.java - Initial Developper : ofabre
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.component.common.listener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;

import org.objectweb.petals.component.common.AbstractComponent;
import org.objectweb.petals.component.common.ComponentConfiguration;
import org.objectweb.petals.component.common.PEtALSCDKException;
import org.objectweb.petals.component.common.util.StringHelper;
import org.objectweb.petals.jbi.descriptor.Consumes;

/**
 * This manager is used to enable a binding component to handle external service
 * call. It manages listeners for external services (start and stop).
 *
 * @author ofabre - EBM Websourcing
 *
 */
public class ExternalListenerManager {

    /**
     *
     */
	private Map<String, List<AbstractExternalListener>> externalListeners;

    /**
     *
     */
	private AbstractComponent component;

    /**
     * Creates a new instance of {@link ExternalListenerManager}
     *
     * @param componentConfiguration
     * @param component
     * @param componentContext
     * @param deliveryChannel
     * @param logger
     */
	public ExternalListenerManager(
			ComponentConfiguration componentConfiguration,
			AbstractComponent component, ComponentContext componentContext,
			DeliveryChannel deliveryChannel, Logger logger) {
		super();
		this.component = component;
		this.externalListeners = new HashMap<String, List<AbstractExternalListener>>();
	}

	/**
	 * Start a listener for each given consumes.
	 *
	 * @param serviceUnitName
	 *            name of the service unit to start
	 * @param consumes
	 *            the consumes nodes defining external services to listen. Not
	 *            null
	 * @throws PEtALSCDKException
	 *             if an error occurs during listener starting
	 */
	public void startListening(String serviceUnitName,
			List<Consumes> consumesList) throws PEtALSCDKException {
		for (Consumes consumes : consumesList) {
			AbstractExternalListener externalListener = createAndInitExternalListener(consumes);
			if (externalListener != null) {
				externalListener.start();
				addExternalListerner(serviceUnitName, externalListener);
			} else {
				throw new PEtALSCDKException(
						"No ExternalListener can be created. Check if you have "
								+ "provided an ExternalListener class "
								+ "name in your component descriptor (jbi.xml)");
			}
		}
	}

    /**
     *
     * @param consumes
     * @return
     * @throws PEtALSCDKException
     */
	private AbstractExternalListener createAndInitExternalListener(
			Consumes consumes) throws PEtALSCDKException {
		AbstractExternalListener externalListener = null;
		String externalListenerClassName = component
				.getComponentConfiguration().getExternalListenerClassName();
		if (!StringHelper.isNullOrEmpty(externalListenerClassName)) {
			try {
				ClassLoader componentClassLoader = this.getClass()
						.getClassLoader();
				Class clazz = componentClassLoader
						.loadClass(externalListenerClassName);
				externalListener = (AbstractExternalListener) clazz
						.newInstance();
			} catch (InstantiationException e) {
				throw new PEtALSCDKException(
						"Can't instanciate ExternaListener", e);
			} catch (IllegalAccessException e) {
				throw new PEtALSCDKException(
						"Can't instanciate ExternaListener", e);
			} catch (ClassNotFoundException e) {
				throw new PEtALSCDKException(
						"Can't find ExternaListener class : "
								+ externalListenerClassName, e);
			}
			externalListener.init(component);
			externalListener.setConsumes(consumes);
			externalListener.init();
		}
		return externalListener;
	}

    /**
     *
     * @param serviceUnitName
     * @param externalListener
     */
	private void addExternalListerner(String serviceUnitName,
			AbstractExternalListener externalListener) {
		List<AbstractExternalListener> externalListenersForSU = externalListeners
				.get(serviceUnitName);
		if (externalListenersForSU == null) {
			externalListenersForSU = new ArrayList<AbstractExternalListener>();
		}
		externalListenersForSU.add(externalListener);
	}

	/**
	 * Stop all external listeners related to the given SU.
	 *
	 * @param serviceUnitName
	 *            name of the service unit to start
	 * @throws PEtALSCDKException
	 *             if an error occurs during listener starting
	 */
	public void stopListening(String serviceUnitName) throws PEtALSCDKException {
		List<AbstractExternalListener> externalListenersForSU = externalListeners
				.get(serviceUnitName);
		if (externalListenersForSU != null) {
			for (AbstractExternalListener listener : externalListenersForSU) {
				listener.stop();
			}
		}
	}
}
