/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.common.listener;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;

import org.objectweb.petals.component.common.AbstractComponent;
import org.objectweb.petals.component.common.IgnoredStatusConstants;
import org.objectweb.petals.component.common.listener.MessageInterceptorProcessor.Phase;
import org.objectweb.petals.component.common.mbean.PerformanceNotification;
import org.objectweb.petals.component.common.su.ServiceUnitManager;
import org.objectweb.petals.component.common.util.MessageExchangeWrapper;
import org.objectweb.petals.component.common.util.MessageExchangeWrapperImpl;
import org.objectweb.petals.jbi.descriptor.Consumes;
import org.objectweb.petals.jbi.descriptor.Provides;

/**
 * Processor for recevied MessageExchanges.
 *
 * @version $Revision: $ $Date: $
 * @since Petals 1.3
 * @author alouis, ofabre - EBM Websourcing
 */
public class MessageExchangeProcessor extends Thread {

    /**
     * The component's delivery channel
     */
    private DeliveryChannel channel;

    /**
     * The component's logger
     */
    private Logger log;

    /**
     * The component's service unit manager
     */
    private ServiceUnitManager suManager;

    private AbstractJBIListener listener;

    private boolean running;

    private AbstractComponent component;

    private JBIListenerManager listenerManager;

    /**
     * Creates a processor that listen to new {@link MessageExchange} on the
     * specified channel. Exchanges are ignored depending on the specified
     * exchange-status to ignore. wait for the specified interval between two
     * 'accept()';
     *
     * @param channel
     * @param log
     * @param suManager
     * @param listener
     * @param component
     */
    public MessageExchangeProcessor(DeliveryChannel channel, Logger log,
            ServiceUnitManager suManager, AbstractJBIListener listener,
            AbstractComponent component, JBIListenerManager listenerManager) {
        super(component.getContext().getComponentName()
                + "-JBI listener thread");
        this.channel = channel;
        this.log = log;
        this.suManager = suManager;
        this.listener = listener;
        this.listenerManager = listenerManager;
        this.component = component;
    }

    /**
     * Launch the component processing of the given message exchange
     *
     * @param exchange
     *            the message exchange to process. Must be non null.
     * @throws InterruptedException
     * @throws JBIException
     *             if an error occured during the message exchange processing
     */
    private void process(final MessageExchangeWrapper exchange)
            throws MessagingException, InterruptedException {
        if (exchange.isActiveStatus()) {
            if (exchange.isProviderRole()) {
                log.finest("Process an exchange as PROVIDER with id : "
                        + exchange.getExchangeId());
                Provides provides = getProvidesFromExchangeEndpoint(exchange);
                listener.setProvides(provides);

                this.listener.getMessageInterceptorProcessor().intercept(
                        exchange, provides, Phase.RECEIVE);

                processAsProvider(exchange);

                listener.setProvides(null);
                channel.send(exchange.getMessageExchange());

            } else {
                log.finest("Process an exchange as CONSUMER with id : "
                        + exchange.getExchangeId());
                Consumes consumes = getConsumesFromExchange(exchange);
                // Notify a synchronizer if exist
                MessageExchangeSynchronizer exchangeSynchronizer = listenerManager
                        .getAsynchronousMessage(exchange.getExchangeId());
                this.listener.getMessageInterceptorProcessor().intercept(
                        exchange, consumes, Phase.RECEIVE);
                if (exchangeSynchronizer != null) {
                    if (!exchangeSynchronizer.isTimeout()) {
                        exchangeSynchronizer.setExchange(exchange);
                        exchangeSynchronizer.notifyExchange();
                    } else {
                        exchange
                                .setError(new Exception(
                                        "The response to an asynchronous exchange "
                                                + "arrived after a timeout occured during accept"));
                        channel.send(exchange.getMessageExchange());
                    }
                } else {
                    listener.setConsumes(consumes);
                    processAsConsumer(exchange);
                    listener.setConsumes(null);
                    channel.send(exchange.getMessageExchange());
                }
            }
        } else {
            log.log(Level.INFO, "No processing on Unactive MEP");
        }
    }

    private Consumes getConsumesFromExchange(MessageExchangeWrapper exchange) {
        ServiceEndpoint endpoint = exchange.getEndpoint();
        return suManager.getConsumesFromDestination(endpoint.getEndpointName(),
                exchange.getService(), exchange.getInterfaceName());
    }

    private void processAsProvider(final MessageExchangeWrapper exchange)
            throws MessagingException {
        if (exchange.isInOnlyPattern()) {
            processInOnlyAsProvider(exchange);
        } else if (exchange.isRobustInOnlyPattern()) {
            processRobustInOnlyAsProvider(exchange);
        } else if (exchange.isInOutPattern()) {
            processInOutAsProvider(exchange);
        } else if (exchange.isInOptionalOutPattern()) {
            processInOptionalOutAsProvider(exchange);
        } else {
            throw new MessagingException(
                    "MessageExchangePattern not recognized :"
                            + exchange.getPattern());
        }
    }

    private void processAsConsumer(final MessageExchangeWrapper exchange)
            throws MessagingException {
        if (exchange.isRobustInOnlyPattern()) {
            processRobustInOnlyAsConsumer(exchange);
        } else if (exchange.isInOutPattern()) {
            processInOutAsConsumer(exchange);
        } else if (exchange.isInOptionalOutPattern()) {
            processInOptionalOutAsConsumer(exchange);
        } else {
            throw new MessagingException(
                    "MessageExchangePattern not recognized :"
                            + exchange.getPattern());
        }
    }

    private void processInOptionalOutAsConsumer(MessageExchangeWrapper exchange)
            throws MessagingException {
        listener.onJBIMessage(exchange);

        // TODO Delete the out message if the it still handled by the message
        // exchange

        // Set Status DONE if no Fault and ACTIVE Status
        if (exchange.getFault() == null && exchange.isActiveStatus()) {
            exchange.setDoneStatus();
        }
    }

    private void processInOutAsConsumer(MessageExchangeWrapper exchange)
            throws MessagingException {
        try {
            listener.onJBIMessage(exchange);

            // TODO Delete the out message if the it still handled by the
            // message
            // exchange
        } finally {
            if (exchange.isActiveStatus()) {
                exchange.setDoneStatus();
            }
        }
    }

    private void processRobustInOnlyAsConsumer(MessageExchangeWrapper exchange)
            throws MessagingException {
        try {
            listener.onJBIMessage(exchange);
        } finally {
            if (exchange.isActiveStatus()) {
                exchange.setDoneStatus();
            }
        }
    }

    /**
     * Process "in-optional-out" MessageExchange
     *
     * @param exchange
     *            the MessageExchange
     * @throws MessagingException
     *             When errors occur during processing
     */
    private void processInOptionalOutAsProvider(MessageExchangeWrapper exchange)
            throws MessagingException {
        if (!ackFaultReception(exchange)) {
            listener.onJBIMessage(exchange);

            // Set Status DONE if no Fault or Out messages and ACTIVE Status
            if (exchange.getFault() == null && exchange.getOutMessage() == null
                    && exchange.isActiveStatus()) {
                exchange.setDoneStatus();
            }
        }
    }

    /**
     * Process "in-only" MessageExchange
     *
     * @param exchange
     *            the MessageExchange
     * @throws MessagingException
     *             When errors occur during processing
     */
    private void processInOnlyAsProvider(final MessageExchangeWrapper exchange)
            throws MessagingException {
        try {
            listener.onJBIMessage(exchange);
        } finally {
            if (exchange.isActiveStatus()) {
                exchange.setDoneStatus();
            }
        }
    }

    /**
     * Process "in-out" MessageExchange
     *
     * @param exchange
     *            the MessageExchange
     * @throws MessagingException
     *             When errors occur during processing
     *
     */
    private void processInOutAsProvider(final MessageExchangeWrapper exchange) {
        listener.onJBIMessage(exchange);
    }

    /**
     * Process "robust-in-only" MessageExchange
     *
     * @param exchange
     *            the MessageExchange
     * @throws MessagingException
     *             When errors occur during processing
     *
     */
    private void processRobustInOnlyAsProvider(
            final MessageExchangeWrapper exchange) throws MessagingException {
        listener.onJBIMessage(exchange);

        // Set Status DONE if no Fault and ACTIVE Status
        if (exchange.getFault() == null && exchange.isActiveStatus()) {
            exchange.setDoneStatus();
        }
    }

    /**
     * If the received exchange is a fault response from the consumer,
     * acknowledge it : status DONE
     *
     * @param exchange
     *            The processed {@link MessageExchange}
     * @return true if the ack. of the Fault is done; false if no fault is
     *         present in the exchange
     * @throws MessagingException
     */
    private boolean ackFaultReception(final MessageExchangeWrapper exchange)
            throws MessagingException {
        boolean result = false;

        if (exchange.getFault() != null) {
            listener.onJBIMessage(exchange);
            exchange.setDoneStatus();
            result = true;
        }
        return result;
    }

    private Provides getProvidesFromExchangeEndpoint(
            final MessageExchangeWrapper exchange) {
        ServiceEndpoint ep = exchange.getEndpoint();
        Provides provides = null;
        if (ep != null) {
            provides = suManager.getProvidesFromEndpoint(ep);
        }
        return provides;
    }

    /**
     * Close the listener. This cause the terminaison of the loop on
     * DeliveryChannel.accept(), and interrupt this listening thread if the
     * thread is blocked on DeliveryChannel.accept();
     */
    public void close() {
        terminate();
        interrupt();
    }

    /**
     * Start the thread listener ( Thread.start()). The listener waits on
     * DeliveryChannel.accept(), treats the received MessageExchange, and waits
     * again on DeliveryChannel.accept().
     */
    public void listen() {
        start();
    }

    /**
     * loop on accept Stop on Interrupted Exception
     */
    public void run() {
        running = true;

        while (running) {

            // receive a message
            MessageExchangeWrapper exchange = null;
            try {
                exchange = new MessageExchangeWrapperImpl(channel.accept());

            } catch (MessagingException e) {
                // search for an interrupted exception
                Throwable cause = e;
                while (cause != null) {
                    cause = cause.getCause();
                    if (cause instanceof InterruptedException) {
                        running = false;
                        log.log(Level.SEVERE,
                                "Thread interrupted, stop listening.");
                    }
                }
                if (running == true) {
                    log.log(Level.SEVERE, e.getMessage(), e);
                }
            }

            // forward the message to a worker
            if (exchange != null) {
                if (!isIgnored(exchange)) {
                    PerformanceNotification performanceNotification = null;
                    try {
                        performanceNotification = this.component
                                .createPerformanceNotificationReceivingFromNMR(
                                        this
                                                .getPerformanceNotificationUserData(),
                                        exchange);

                        process(exchange);

                    } catch (Throwable e) {
                        log.log(Level.SEVERE, e.getMessage(), e);
                        exchange.setError(new Exception(e));
                        try {
                            channel.send(exchange.getMessageExchange());
                        } catch (MessagingException e1) {
                            log.log(Level.SEVERE, "Can't send Provider Error "
                                    + "MessageExchange to consumer", e);
                        }
                    } finally {
                        this.component
                                .sendPerformanceNotification(performanceNotification);
                    }
                }
            }
        }
    }

    /**
     * This method should be overriden if you wish to set custom user data to
     * your performance notification.
     *
     * @return Perfomance notification user data.
     */
    // TODO: Use the caller name as user data.
    protected String getPerformanceNotificationUserData() {
        return "";
    }

    /**
     * Terminate the listening. On the next MessageExchange treated after the
     * DeliveryChannel.accept(), the listener will stop waiting on
     * DeliveryChannel.accept().
     */
    public void terminate() {
        running = false;
    }

    /**
     * depending on the IgnoredStatus, tells if the exchange has to be ignored.
     * If the exchange is null, return true.
     *
     * @param exchange
     * @return
     */
    protected boolean isIgnored(final MessageExchangeWrapper exchange) {
        boolean result = false;

        if (exchange.getMessageExchange() == null) {
            result = true;
        } else {
            IgnoredStatusConstants status = component
                    .getComponentConfiguration().getIgnoredStatus();
            if (exchange.isDoneStatus()) {
                result = IgnoredStatusConstants.DONE_IGNORED == status
                        || IgnoredStatusConstants.DONE_AND_ERROR_IGNORED == status;
            } else if (exchange.isErrorStatus()) {
                result = IgnoredStatusConstants.ERROR_IGNORED == status
                        || IgnoredStatusConstants.DONE_AND_ERROR_IGNORED == status;
            }
        }
        return result;
    }
}
