/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.common.mbean;

import java.util.HashMap;
import java.util.Iterator;

import javax.jbi.messaging.MessageExchange;
import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import javax.management.DynamicMBean;
import javax.management.InvalidAttributeValueException;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanConstructorInfo;
import javax.management.MBeanException;
import javax.management.MBeanInfo;
import javax.management.MBeanNotificationInfo;
import javax.management.MBeanOperationInfo;
import javax.management.Notification;
import javax.management.NotificationBroadcasterSupport;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.objectweb.petals.component.common.ComponentConfiguration;
import org.objectweb.petals.component.common.util.IDGenerator;

/**
 * The MBean used to manage the component via JMX.
 *
 * @author Christophe DENEUX - Capgemini Sud
 *
 */
/**
 * @author CD
 *
 */
public class PerformanceNotifierMBean extends NotificationBroadcasterSupport implements DynamicMBean {
	
    // Name of the normalized message containing the performance notification UID. 
    public static final String PERF_NOTIF_NMR_MSG_ATTR = "petals:perfNotifUID";

    // JMX Attributes names
	private static final String ATTR_NOTIFICATIONS_ENABLED = "notificationsEnabled";
	private static final String ATTR_NOTIFICATIONS_STEP = "notificationsStep";
	
    private static final String ATTR_NOTIFICATIONS_VALUES = "performanceNotificationData";

	// JMX sequence numbers and their mutex
    private PerformanceNotificationCounter counters = null;
    private long jmxGlobalSeqNum = 0;
    private Object jmxGlobalSeqNumObj = new Object();
	
	private ComponentConfiguration componentConfiguration = null;
	
	/**
     * JMX Performance notifier object name
     */
    private ObjectName jmxPerformanceNotifierName = null;

    /**
	 * Constructor
	 */
	public PerformanceNotifierMBean(ComponentConfiguration componentConfiguration, ObjectName jmxPerformanceNotifierName) {
		this.componentConfiguration = componentConfiguration;
		this.jmxPerformanceNotifierName = jmxPerformanceNotifierName;
		this.counters = new PerformanceNotificationCounter(this.componentConfiguration.getPerformanceNotificationsStep());
	}

	public ObjectName getJmxPerformanceNotifierName() {
		return jmxPerformanceNotifierName;
	}

	/**
     * create a performance notification
     */
    public final PerformanceNotification createPerformanceNotifications(
            final long timestamp, final String owner, final String typeNotif, final String userData, MessageExchange msgExchange) {
        
        PerformanceNotification performanceNotification = null;
        if (counters.inc(typeNotif)) {
            // We try to find a performance notification message exists in the exchange, or create a new one
            String perfUID = null;
            if (msgExchange != null) {
                // we try to find a performance notification UID
                perfUID = (String)msgExchange.getProperty(PERF_NOTIF_NMR_MSG_ATTR);
            }
            
            if (perfUID == null) {
                perfUID = IDGenerator.getNewID();
                if (msgExchange != null) {
                    msgExchange.setProperty(PERF_NOTIF_NMR_MSG_ATTR, perfUID);
                }
            }
            
            synchronized(jmxGlobalSeqNumObj) {
                performanceNotification = new PerformanceNotification(timestamp, owner, typeNotif, userData, ++jmxGlobalSeqNum, perfUID);
            }
        }
        
        return performanceNotification;
    }

    /**
	 * Publish the performance notification
	 * @param notification
	 */
	public final void send(final PerformanceNotification notification) {
	    
	    if (notification != null) {
	        notification.end();

	        // Notification creation
            Notification jmxNotification = new Notification(ATTR_NOTIFICATIONS_VALUES, 
                                        this.getClass().getName(), 
                                        notification.getGlobalSeqNum());

            String[] userData = {
                    notification.getUID(),
                    Long.toString(notification.getTimestamp()),
                    notification.getOwner(),
                    notification.getType(),
                    Long.toString(notification.getDuration()),
                    notification.getUserData()};

            jmxNotification.setUserData(userData);

            // Notification sending
            this.sendNotification(jmxNotification);
	    }
		
	}

/*
 * Interface DynamicMBean
 */
    
    
    /*
     * (non-Javadoc)
     * @see javax.management.DynamicMBean#getMBeanInfo()
     */
    public MBeanInfo getMBeanInfo() {
    	// Generation liste des attributs
        MBeanAttributeInfo[] mbaAttributes = {
				new MBeanAttributeInfo(ATTR_NOTIFICATIONS_STEP,
						"java.lang.Integer",
						"Notification step",
						true,
						true,
						false),
				new MBeanAttributeInfo(ATTR_NOTIFICATIONS_ENABLED,
						"java.lang.Boolean",
						"Notification activation",
						true,
						true,
						false) };
    	// Generation MBeanInfo object
        return new MBeanInfo(this.getClass().getName(),
                                        "JMX Performance Notifier",
                                        mbaAttributes,
                                        new MBeanConstructorInfo[0],
                                        new MBeanOperationInfo[0],
                                        new MBeanNotificationInfo[0]);
    }
    
    
    /*
     * (non-Javadoc)
     * @see javax.management.DynamicMBean#getAttributes(java.lang.String[])
     */
    public AttributeList getAttributes(String[] arg0) {
    	// TODO Auto-generated method stub
        AttributeList alAttributesList;
        int iAtbIndex;
        String sAtbId;
        Object oAtbValue;

        // Genration liste des attributs associ�s a leur valeur
        alAttributesList = new AttributeList();
        // Boucle de parcours des attributs publies
        for (iAtbIndex = 0; iAtbIndex < arg0.length; iAtbIndex++) {
            oAtbValue = null;
            sAtbId = arg0[iAtbIndex];
            try {
                oAtbValue = this.getAttribute(sAtbId);
                alAttributesList.add(new Attribute(sAtbId, oAtbValue));
            } catch (AttributeNotFoundException anfe) {} 
            catch (ReflectionException anfe) {} 
            catch (MBeanException mbe) {}
        }
        return alAttributesList;
    }
    
    
    /*
     * (non-Javadoc)
     * @see javax.management.DynamicMBean#getAttribute(java.lang.String)
     */
    public Object getAttribute(String arg0) throws AttributeNotFoundException, 
    												MBeanException, 
    												ReflectionException {
    	// TODO Auto-generated method stub
        if ( arg0.equals(ATTR_NOTIFICATIONS_STEP)) {
               return new Integer(this.componentConfiguration.getPerformanceNotificationsStep());
        } else {
            if ( arg0.equals(ATTR_NOTIFICATIONS_ENABLED)) {
                return new Boolean(this.componentConfiguration.isPerformanceNotificationsEnabled());
            } else {
                throw new AttributeNotFoundException(arg0);
            }
        }
    }
    
    
    /*
     * (non-Javadoc)
     * @see javax.management.DynamicMBean#setAttributes(javax.management.AttributeList)
     */
    public AttributeList setAttributes(AttributeList arg0) {
    	// TODO Auto-generated method stub
        Iterator itAtbIterator;
        Attribute atbCurrent;

        for (itAtbIterator = arg0.iterator(); itAtbIterator.hasNext();) {
            atbCurrent = (Attribute) itAtbIterator.next();
            try {
                this.setAttribute(atbCurrent);          
            } catch (AttributeNotFoundException anfe) {
            }
            catch (MBeanException mbe) {}
            catch (InvalidAttributeValueException ivae) {}
            catch (ReflectionException re) {}
        }
        return arg0;
    }
    
    
    /*
     * (non-Javadoc)
     * @see javax.management.DynamicMBean#setAttribute(javax.management.Attribute)
     */
    public void setAttribute(Attribute arg0) throws AttributeNotFoundException, 
    													InvalidAttributeValueException, 
    													MBeanException, 
    													ReflectionException {
    	if ( arg0.getName().equals(ATTR_NOTIFICATIONS_ENABLED) == true ) {
    		this.componentConfiguration.setPerformanceNotificationsEnabled(((Boolean) arg0.getValue()).booleanValue());
    	} else {
            if ( arg0.getName().equals(ATTR_NOTIFICATIONS_STEP) == true ) {
                this.componentConfiguration.setPerformanceNotificationsStep(((Integer) arg0.getValue()).intValue());
                this.counters.setStep(this.componentConfiguration.getPerformanceNotificationsStep());
            } else {
                throw new AttributeNotFoundException(arg0.getName());
            }
    	}
    }
    
    /*
     * (non-Javadoc)
     * @see javax.management.DynamicMBean#invoke(java.lang.String, java.lang.Object[], java.lang.String[])
     */
    public Object invoke(String arg0, Object[] arg1, String[] arg2) throws MBeanException, 
    																		ReflectionException {
    	// TODO Auto-generated method stub
    	return null;
    }
    
}
