
package org.objectweb.petals.component.common.util;

import java.net.URI;
import java.util.Map;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.transform.Source;

public interface MessageExchangeWrapper extends MessageExchange {

    /**
     * Returns the initial message exchange
     *
     * @return
     */
    public MessageExchange getMessageExchange();

    /**
     * Put a Source object in the out message of the message exchange. If the
     * out normalized message is null, it is created here.
     *
     * @param outContent
     *            the String content to put in the out message
     * @throws MessagingException
     *             if error occured during out message creation
     */
    void setOutMessageContent(final String outContent)
            throws MessagingException;

    /**
     * Put a Source object in the out message of the message exchange. If the
     * out normalized message is null, it is created here.
     *
     * @param outContent
     *            the Source content to put in the out message
     * @throws MessagingException
     *             if error occured during out message creation
     */
    void setOutMessageContent(final Source outContent)
            throws MessagingException;

    /**
     * Put a Source object in the in message of the message exchange. If the in
     * normalized message is null, it is created here.
     *
     * @param inContent
     *            the String content to put in the in message
     * @throws MessagingException
     *             if error occured during in message creation
     */
    void setInMessageContent(final String inContent) throws MessagingException;

    /**
     * Put a Source object in the in message of the message exchange. If the in
     * normalized message is null, it is created here.
     *
     * @param inContent
     *            the Source content to put in the in message
     * @throws MessagingException
     *             if error occured during in message creation
     */
    void setInMessageContent(final Source inContent) throws MessagingException;

    /**
     * Get the content of the "in" message as a String
     *
     * @return the String represention of the "in" message content
     * @throws MessagingException
     *             if error occured during String creation from Source or if
     *             "in" message is null
     */
    String getInMessageContentAsString() throws MessagingException;

    /**
     * Get the content of the "in" message as a Source
     *
     * @return the "in" message content
     * @throws MessagingException
     *             if "in" message is null
     */
    Source getInMessageContentAsSource() throws MessagingException;

    /**
     * Get the content of the "out" message as a String
     *
     * @return the String represention of the "out" message content
     * @throws MessagingException
     *             if error occured during String creation from Source or if
     *             "out" message is null
     */
    String getOutMessageContentAsString() throws MessagingException;

    /**
     * Get the content of the "out" message as a Source
     *
     * @return the "out" message content
     * @throws MessagingException
     *             if "out" message is null
     */
    Source getOutMessageContentAsSource() throws MessagingException;

    /**
     * Get all the input attachments in a {@link Set}
     *
     * @param exchange
     * @return
     */
    Set<DataHandler> getInMessageAttachments() throws MessagingException;

    /**
     * Get the message exchange Pattern
     *
     * @return the URI pattern. Can't be null
     * @throws MessagingException
     *             if no message exchange pattern is set
     */
    URI getExchangePattern() throws MessagingException;

    /**
     * Get the operation name (local part of the operation QName)
     *
     * @return the operation name
     * @throws MessagingException
     *             if no Operation QName is set
     */
    String getOperationName() throws MessagingException;

    /**
     * Get the endpoint name specified in the message exchange
     *
     * @return the endpoint name
     * @throws MessagingException
     *             if no Endpoint is set
     */
    String getEndpointName() throws MessagingException;

    /**
     * Extract incoming NormalizedMessage from given MessageExchange
     *
     * @return the incoming NormalizedMessage
     *
     */
    NormalizedMessage getInMessage();

    /**
     * Set the input message
     *
     * @param msg
     * @throws MessagingException
     */
    void setInMessage(final NormalizedMessage msg) throws MessagingException;

    /**
     * Extract outgoing NormalizedMessage from given MessageExchange
     *
     * @return the outgoing NormalizedMessage
     *
     */
    NormalizedMessage getOutMessage();

    /**
     * Set the output message
     *
     * @param msg
     * @throws MessagingException
     */
    void setOutMessage(final NormalizedMessage msg) throws MessagingException;

    /**
     * Create a {@link Fault} with the given {@link Exception} and set it in the
     * {@link MessageExchange}
     *
     * @param e
     *            the {@link Exception} to transform in {@link Fault}
     * @throws MessagingException
     */
    void setFault(Exception e) throws MessagingException;

    /**
     * Put a Map of attachements in the out message of the message exchange. If
     * the out normalized message is null, it is created here.
     *
     * @param attachements
     *            the attachements to put in the out message
     * @throws MessagingException
     *             if error occured during out message creation
     */
    void setOutMessageAttachements(Map<String, DataHandler> attachements)
            throws MessagingException;

    /**
     * Set the message exchange status to DONE
     *
     * @throws MessagingException
     */
    public void setDoneStatus() throws MessagingException;

    /**
     * Test if the message status is DONE
     *
     * @return
     */
    public boolean isDoneStatus();

    /**
     * Set the message exchange status to Error
     *
     * @throws MessagingException
     */
    public void setErrorStatus() throws MessagingException;

    /**
     * Test if the message status is ERROR
     *
     * @return
     */
    public boolean isErrorStatus();

    /**
     * Set the message exchange status to ACTIVE
     *
     * @throws MessagingException
     */
    public void setActiveStatus() throws MessagingException;

    /**
     * Test if the message status is ACTIVE
     *
     * @return
     */
    public boolean isActiveStatus();

    /**
     *
     * @return true is the message role is provider
     */
    public boolean isProviderRole();

    /**
     *
     * @return true is the message role is consumer
     */
    public boolean isConsumerRole();

    /**
     *
     * @return
     */
    public boolean isInOnlyPattern();

    /**
     *
     * @return
     */
    public boolean isRobustInOnlyPattern();

    /**
     *
     * @return
     */
    public boolean isInOutPattern();

    /**
     *
     * @return
     */
    public boolean isInOptionalOutPattern();

}
