/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.common.util;

import java.net.URI;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.objectweb.petals.component.common.MEPConstants;
import org.objectweb.petals.component.common.PEtALSCDKException;

/**
 * This is a Message Exchange Wrapper
 *
 * @author ofabre
 *
 */
public class MessageExchangeWrapperImpl implements MessageExchangeWrapper {

    private MessageExchange exchange;

    private static final String IN_MESSAGE_NAME = "in";

    private static final String OUT_MESSAGE_NAME = "out";

    public MessageExchangeWrapperImpl(MessageExchange exchange) {
        super();
        this.exchange = exchange;
    }

    public MessageExchange getMessageExchange() {
        return exchange;
    }

    public Fault createFault() throws MessagingException {
        return exchange.createFault();
    }

    public NormalizedMessage createMessage() throws MessagingException {
        return exchange.createMessage();
    }

    public ServiceEndpoint getEndpoint() {
        return exchange.getEndpoint();
    }

    public Exception getError() {
        return exchange.getError();
    }

    public String getExchangeId() {
        return exchange.getExchangeId();
    }

    public Fault getFault() {
        return exchange.getFault();
    }

    public QName getInterfaceName() {
        return exchange.getInterfaceName();
    }

    public NormalizedMessage getMessage(String name) {
        return exchange.getMessage(name);
    }

    public QName getOperation() {
        return exchange.getOperation();
    }

    public URI getPattern() {
        return exchange.getPattern();
    }

    public Object getProperty(String name) {
        return exchange.getProperty(name);
    }

    public Set getPropertyNames() {
        return exchange.getPropertyNames();
    }

    public Role getRole() {
        return exchange.getRole();
    }

    public QName getService() {
        return exchange.getService();
    }

    public ExchangeStatus getStatus() {
        return exchange.getStatus();
    }

    public boolean isTransacted() {
        return exchange.isTransacted();
    }

    public void setEndpoint(ServiceEndpoint endpoint) {
        exchange.setEndpoint(endpoint);
    }

    /**
     * Put the given exception as Error part of the message exchange. Allowed
     * exception types are : {@link java.*}, {@link javax.jbi.JBIException},
     * org.w3c.* and org.xml.*.
     *
     */
    public void setError(Exception error) {
        String packageName = error.getClass().getPackage().getName();
        if (packageName.startsWith("java") || packageName.startsWith("org.w3c")
                || packageName.startsWith("org.xml")) {
            exchange.setError(error);
        } else {
            throw new IllegalArgumentException("Exception of type ("
                    + packageName + ") are not supported by the JBI container");
        }
    }

    public void setFault(Fault fault) throws MessagingException {
        exchange.setFault(fault);
    }

    public void setInterfaceName(QName interfaceName) {
        exchange.setInterfaceName(interfaceName);
    }

    public void setMessage(NormalizedMessage msg, String name)
            throws MessagingException {
        exchange.setMessage(msg, name);
    }

    public void setOperation(QName name) {
        exchange.setOperation(name);
    }

    public void setProperty(String name, Object obj) {
        exchange.setProperty(name, obj);
    }

    public void setService(QName service) {
        exchange.setService(service);
    }

    public void setStatus(ExchangeStatus status) throws MessagingException {
        exchange.setStatus(status);
    }

    public void setOutMessageContent(String outContent)
            throws MessagingException {
        try {
            NormalizedMessage outMess = getOutMessage();
            if (outMess == null) {
                outMess = exchange.createMessage();
            }
            outMess.setContent(SourceHelper.createSource(outContent));
            setOutMessage(outMess);
        } catch (PEtALSCDKException e) {
            throw new MessagingException("Error creating response message");
        }
    }

    public void setOutMessageContent(Source outContent)
            throws MessagingException {
        NormalizedMessage outMess = getOutMessage();
        if (outMess == null) {
            outMess = exchange.createMessage();
        }
        outMess.setContent(outContent);
        setOutMessage(outMess);
    }

    public void setInMessageContent(String inContent) throws MessagingException {
        try {
            NormalizedMessage inMess = getInMessage();
            if (inMess == null) {
                inMess = exchange.createMessage();
            }
            inMess.setContent(SourceHelper.createSource(inContent));
            setInMessage(inMess);
        } catch (PEtALSCDKException e) {
            throw new MessagingException("Error creating request message");
        }
    }

    public void setInMessageContent(Source inContent) throws MessagingException {
        NormalizedMessage inMess = getInMessage();
        if (inMess == null) {
            inMess = exchange.createMessage();
        }
        inMess.setContent(inContent);
        setInMessage(inMess);
    }

    public void setOutMessageAttachements(Map<String, DataHandler> attachements)
            throws MessagingException {
        NormalizedMessage outMess = getOutMessage();
        if (outMess == null) {
            outMess = exchange.createMessage();
        }
        for (String attachementId : attachements.keySet()) {
            outMess.addAttachment(attachementId, attachements
                    .get(attachementId));
        }
        setOutMessage(outMess);
    }

    public String getInMessageContentAsString() throws MessagingException {
        NormalizedMessage inMessage = getInMessage();
        if (inMessage == null) {
            throw new MessagingException("In message must not be null");
        }
        Source content = inMessage.getContent();
        String stringContent = null;
        try {
            stringContent = SourceHelper.createString(content);
        } catch (PEtALSCDKException e) {
            throw new MessagingException("Error creating string from source", e);
        }
        return stringContent;
    }

    public Source getInMessageContentAsSource() throws MessagingException {
        NormalizedMessage inMessage = getInMessage();
        if (inMessage == null) {
            throw new MessagingException("In message must not be null");
        }
        return inMessage.getContent();
    }

    public String getOutMessageContentAsString() throws MessagingException {
        NormalizedMessage outMessage = getOutMessage();
        if (outMessage == null) {
            throw new MessagingException("Out message must not be null");
        }
        Source content = outMessage.getContent();
        String stringContent = null;
        try {
            stringContent = SourceHelper.createString(content);
        } catch (PEtALSCDKException e) {
            throw new MessagingException("Error creating string from source", e);
        }
        return stringContent;
    }

    public Source getOutMessageContentAsSource() throws MessagingException {
        NormalizedMessage outMessage = getOutMessage();
        if (outMessage == null) {
            throw new MessagingException("Out message must not be null");
        }
        return outMessage.getContent();
    }

    public Set<DataHandler> getInMessageAttachments() throws MessagingException {
        Set<DataHandler> attachments = new HashSet<DataHandler>();

        NormalizedMessage inMessage = getInMessage();
        if (inMessage == null) {
            throw new MessagingException("In message must not be null");
        }

        Set names = inMessage.getAttachmentNames();
        for (Object attachId : names) {
            DataHandler dh = inMessage.getAttachment((String) attachId);
            if (dh != null)
                attachments.add(dh);
        }
        return attachments;
    }

    public URI getExchangePattern() throws MessagingException {
        URI pattern = exchange.getPattern();
        if (pattern == null) {
            throw new MessagingException(
                    "You must specify a message exchange pattern in the message exchange");
        }
        return pattern;
    }

    public String getOperationName() throws MessagingException {
        QName operation = exchange.getOperation();
        if (operation == null) {
            throw new MessagingException(
                    "You must specify an operation the message exchange");
        }
        return operation.getLocalPart();
    }

    public String getEndpointName() throws MessagingException {
        ServiceEndpoint ep = exchange.getEndpoint();
        if (ep == null) {
            throw new MessagingException(
                    "You must specify an endpoint in the message exchange");
        }
        return ep.getEndpointName();
    }

    public NormalizedMessage getInMessage() {
        return exchange.getMessage(IN_MESSAGE_NAME);
    }

    public void setInMessage(final NormalizedMessage msg)
            throws MessagingException {
        exchange.setMessage(msg, IN_MESSAGE_NAME);
    }

    public NormalizedMessage getOutMessage() {
        return exchange.getMessage(OUT_MESSAGE_NAME);
    }

    public void setOutMessage(final NormalizedMessage msg)
            throws MessagingException {
        exchange.setMessage(msg, OUT_MESSAGE_NAME);
    }

    /* ### STATUS ### */

    public void setDoneStatus() throws MessagingException {
        setStatus(ExchangeStatus.DONE);
    }

    public boolean isDoneStatus() {
        return (ExchangeStatus.DONE.equals(getStatus()));
    }

    public void setErrorStatus() throws MessagingException {
        setStatus(ExchangeStatus.ERROR);
    }

    public boolean isErrorStatus() {
        return (ExchangeStatus.ERROR.equals(getStatus()));
    }

    public void setActiveStatus() throws MessagingException {
        setStatus(ExchangeStatus.ACTIVE);
    }

    public boolean isActiveStatus() {
        return (ExchangeStatus.ACTIVE.equals(getStatus()));
    }

    /* ### ROLE ### */

    public boolean isProviderRole() {
        return (MessageExchange.Role.PROVIDER.equals(getRole()));
    }

    public boolean isConsumerRole() {
        return (MessageExchange.Role.CONSUMER.equals(getRole()));
    }

    /* ### PATTERN ### */

    public boolean isInOnlyPattern() {
        return (MEPConstants.IN_ONLY_PATTERN.value().equals(exchange
                .getPattern()));
    }

    public boolean isRobustInOnlyPattern() {
        return (MEPConstants.ROBUST_IN_ONLY_PATTERN.value().equals(exchange
                .getPattern()));
    }

    public boolean isInOutPattern() {
        return (MEPConstants.IN_OUT_PATTERN.value().equals(exchange
                .getPattern()));
    }

    public boolean isInOptionalOutPattern() {
        return (MEPConstants.IN_OPTIONAL_OUT_PATTERN.value().equals(exchange
                .getPattern()));
    }

    public void setFault(Exception e) throws MessagingException {
        Fault f = exchange.createFault();
        String faultString = SourceHelper.createSoapFault(e, "0");
        Source content;
        try {
            content = SourceHelper.createSource(faultString);
        } catch (PEtALSCDKException e1) {
            throw new MessagingException(
                    "Can't create Fault content from Exception", e1);
        }
        f.setContent(content);
        exchange.setFault(f);
    }
}
