/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SourceHelper.java 94 2006-03-26 17:11:05Z alouis $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.common.util;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Properties;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.objectweb.petals.component.common.PEtALSCDKException;

/**
 * Sample class used to manage the mapping between
 * <code>javax.xml.transform.Source</code> from String. the mapping is a
 * quicly "string" -> "<content>string</content>" conversion.
 * Initial developer(s): Adrien LOUIS
 * @author Adrien LOUIS - EBM WebSourcing
 */
public final class SourceHelper {

    private static final String START_TAG = "<content>";

    private static final String END_TAG = "</content>";

    private static final int SOAP_FAULT_BUFFER_SIZE = 2048;

    private static Transformer transformer;

    /**
     * Creates a new instance of {@link SourceHelper}
     *
     */
    private SourceHelper() {
        super();
    }

    /**
     * Create a Source from String adding start and end tags.
     *
     * @param msg
     * @return
     * @throws PEtALSCDKException
     */
    public static Source createContentSource(final String msg)
        throws PEtALSCDKException {
        return createSource(START_TAG + msg + END_TAG);
    }

    /**
     * Creates an XML fault with the stacktrace, message and the specified code
     *
     * @param e
     *            exception, used to provide faultstring and detail
     * @param code
     *            code of the fault
     * @return
     */
    public static String createSoapFault(final Throwable e, final String code) {
        StringBuffer s = new StringBuffer(SOAP_FAULT_BUFFER_SIZE);
        s.append("<s:Fault xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\">"
                + "<s:faultcode>");
        if (code != null) {
            s.append(code);
        }
        s.append("</s:faultcode><s:faultstring>");
        s.append(e.getMessage());
        s.append("</s:faultstring><s:detail><![CDATA[");
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        e.printStackTrace(pw);
        s.append(sw);
        s.append("]]></s:detail></s:Fault>");
        return s.toString();
    }

    /**
     * Create a Source from the String. The string must be in UTF-8 format,
     * otherwise, the Source is not created.
     *
     * @param msg
     *            an UTF-8 String
     * @return the Source built from the String, null if not an UTF-8
     */
    public static Source createSource(final String msg)
            throws PEtALSCDKException {
        StreamSource source = new StreamSource();
        try {
            byte[] msgByte = msg.getBytes("UTF-8");
            ByteArrayInputStream in = new ByteArrayInputStream(msgByte);
            source.setInputStream(in);
        } catch (IOException e) {
            throw new PEtALSCDKException(e);
        }
        return source;
    }

    /**
     * Create a String from a Source. The String will be in UTF-8.
     *
     * @param s
     * @return
     * @throws PEtALSCDKException
     */
    public static String createString(final Source s)
        throws PEtALSCDKException {

        String result = null;
        try {
            if (s instanceof StreamSource) {
                StreamSource ss = (StreamSource) s;
                ss.getInputStream().reset();
                byte[] buf = new byte[ss.getInputStream().available()];
                BufferedInputStream bis = new BufferedInputStream(ss
                        .getInputStream());
                bis.read(buf);
                result = new String(buf);
                ss.getInputStream().reset();
            } else {
                StringWriter buffer = new StringWriter();
                Result sresult = new StreamResult(buffer);
                Transformer transform = getTransformer();
                transform.transform(s, sresult);
                result = buffer.toString();
            }
        } catch (IOException e) {
            throw new PEtALSCDKException(e);
        } catch (TransformerConfigurationException e) {
            throw new PEtALSCDKException(e);
        } catch (TransformerException e) {
            throw new PEtALSCDKException(e);
        }
        return result;
    }

    /**
     *
     * @return
     * @throws TransformerConfigurationException
     */
    protected static Transformer getTransformer()
            throws TransformerConfigurationException {
        synchronized (transformer) {
            if (transformer == null) {
                transformer = TransformerFactory.newInstance().newTransformer();
                Properties props = new Properties();
                props.put(OutputKeys.OMIT_XML_DECLARATION, "yes");
                transformer.setOutputProperties(props);
            }
        }
        return transformer;
    }

    /**
     * Create a String from a Source.
     *
     * @param s
     * @return the String in UTF-8 representation without the start and end tags
     * @throws PEtALSCDKException
     */
    public static String createStringContent(final Source s)
        throws PEtALSCDKException {
        String content = createString(s);
        int index = content.indexOf(START_TAG);
        String result = content.substring(index + START_TAG.length());
        return result.substring(0, result.indexOf(END_TAG));
    }

    /**
     * @param args
     */
    public static void main(String[] args) {

    	Source source=null;
    	try {
			source = createSource("<in><code>Hervé</code><nom/></in>");
		} catch (PEtALSCDKException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		try {
			System.out.println(createString(source));
		} catch (PEtALSCDKException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

    }

}
