/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.common.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.wsdl.BindingOperation;
import javax.wsdl.Definition;
import javax.wsdl.Port;
import javax.wsdl.Service;
import javax.wsdl.WSDLException;
import javax.wsdl.extensions.ExtensibilityElement;
import javax.wsdl.extensions.http.HTTPAddress;
import javax.wsdl.extensions.soap.SOAPAddress;
import javax.wsdl.extensions.soap12.SOAP12Address;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;
import javax.wsdl.xml.WSDLWriter;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.TransformerException;

import org.objectweb.petals.component.common.PEtALSCDKException;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.ibm.wsdl.xml.WSDLReaderImpl;
import com.ibm.wsdl.xml.WSDLWriterImpl;

/**
 * Util class to manipulate WSDL Document or Definition
 *
 * @author wjoseph - eBMWebsourcing
 */
public final class WSDLHelper {

    /**
     * Creates a new instance of {@link WSDLHelper}
     *
     */
    private WSDLHelper() {
    }

    /**
     * Change the location attribute of the service node to
     * "/axis2/services/serviceName"
     *
     * @param serviceName
     *            Name of the service
     * @param serviceDesc
     *            Dom representation of the service description
     */
    public static void changeServiceAddressInWSDL(final QName serviceName,
            final Definition serviceDef, final String prefix) {
        SOAPAddress address = findServiceAddress(serviceDef, serviceName);
        if (address != null) {
            String serviceAddress = prefix + serviceName.getLocalPart();
            address.setLocationURI(serviceAddress);
        }
    }

    /**
     * Change the location attribute of the service node to the given address
     *
     * @param serviceName
     *            Name of the service
     * @param serviceDesc
     *            Dom representation of the service description
     * @param webServiceAddress
     *            new service address
     * @throws WSDLException
     * @throws TransformerException
     */
    public static Document changeServiceAddressInWSDL(final QName serviceName,
            final Document serviceDesc, final String webServiceAddress)
            throws WSDLException, TransformerException {
        String docString = XMLUtil.createStringFromDOMDocument(serviceDesc);
        Document tmpDoc = XMLUtil.createDocumentFromString(docString);
        Definition serviceDef = createDefinitionFromDocument(tmpDoc);
        changeServiceAddressInWSDL(serviceName, serviceDef, webServiceAddress);
        return createDocumentFromDefinition(serviceDef);
    }

    /**
     * Creates a {@link Definition} from a {@link Document}
     *
     * @param doc
     * @return
     * @throws WSDLException
     */
    public static Definition createDefinitionFromDocument(final Document doc)
            throws WSDLException {
        WSDLReader reader = new WSDLReaderImpl();
        return reader.readWSDL(null, doc);
    }

    /**
     * Create a dom document from a File
     *
     * @param wsdlFile
     *            file
     * @return dom document
     */
    public static Document createDocumentFromWSDL(final File wsdlFile) {
        URL url = null;
        Document doc = null;
        try {
            url = wsdlFile.toURL();
            doc = createDocumentFromWSDL(url);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return doc;
    }

    /**
     * Get a {@link Document} from a wsdl located on the given {@link URL}.
     *
     * @param url
     * @return null if there is an error getting the wsdl or if the parsing
     *         failed
     * @throws Exception
     *             the Document can not be created
     */
    @SuppressWarnings("unchecked")
    public static Document createDocumentFromWSDL(final URL url)
            throws PEtALSCDKException {
        Document doc = null;
        URLConnection connection = null;
        InputStream is = null;

        try {
            connection = url.openConnection();
            is = connection.getInputStream();

            DocumentBuilderFactory factory = DocumentBuilderFactory
                    .newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();

            doc = builder.parse(is);
            doc.normalize();
        } catch (Exception e) {
            throw new PEtALSCDKException("Can not get WSDL document for URL "
                    + url, e);
        } finally {
            try {
                if (is != null) {
                    is.close();
                }
            } catch (IOException e) {
                // Nothing to do
            }
        }
        return doc;
    }

    /**
     * Find the service address for the given service name.
     *
     * @param serviceDef
     *            the WSDL definition
     * @param serviceName
     *            the qualified service name
     * @return {@link SOAPAddress} if service found in definition, else null
     */
    @SuppressWarnings("unchecked")
    public static SOAPAddress findServiceAddress(Definition serviceDef,
            QName serviceName) {
        SOAPAddress result = null;
        if (serviceDef != null) {
            Service service = serviceDef.getService(serviceName);
            if (service != null) {
                Map<String, Port> ports = service.getPorts();
                for (Port port : ports.values()) {
                    List<ExtensibilityElement> extensibilityElements = port
                            .getExtensibilityElements();
                    for (ExtensibilityElement element : extensibilityElements) {
                        // The SOAP binding is an extensibility element of
                        // Port
                        if (element instanceof SOAPAddress) {
                            result = (SOAPAddress) element;
                            break;
                        }
                    }
                    if (result != null) {
                        break;
                    }
                }
            }
        }
        return result;
    }

    /**
     * Returns the service's port name contained in a WSDL 1.0 document
     *
     * @param serviceDesc
     *            The service description document
     * @return The service port name attribute or null if not found
     */
    @SuppressWarnings("unchecked")
    public static String getPortNameFromWSDLDefinition(Definition serviceDef,
            QName serviceName) {
        String result = null;
        Service service = serviceDef.getService(serviceName);
        Map<String, Port> ports = service.getPorts();
        for (Port port : ports.values()) {
            result = port.getName();
        }
        return result;
    }

    /**
     * Returns the service's port name contained in a WSDL 1.0 document
     *
     * @param serviceDesc
     *            The service description document
     * @return The service port name attribute or null if not found
     * @throws WSDLException
     * @throws TransformerException
     */
    public static String getPortNameFromWSDLDocument(
            final Document serviceDesc, final QName serviceName)
            throws WSDLException, TransformerException {
        String docString = XMLUtil.createStringFromDOMDocument(serviceDesc);
        Document tmpDoc = XMLUtil.createDocumentFromString(docString);
        Definition serviceDef = createDefinitionFromDocument(tmpDoc);
        return getPortNameFromWSDLDefinition(serviceDef, serviceName);
    }

    /**
     * Returns the service address contained in a WSDL 1.0 document
     *
     * @param serviceDesc
     *            The service description document
     * @return The service address attribute or null if not found
     */
    public static String getServiceAddressFromWSDLDefinition(
            final Definition serviceDef, final QName serviceName) {
        String result = null;
        SOAPAddress address = findServiceAddress(serviceDef, serviceName);
        if (address != null) {
            result = address.getLocationURI();
        }
        return result;
    }

    /**
     * Returns the service address contained in a WSDL 1.0 document
     *
     * @param serviceDesc
     *            The service description document
     * @return The service address attribute or null if not found
     * @throws WSDLException
     * @throws TransformerException
     */
    public static String getServiceAddressFromWSDLDocument(
            final Document serviceDesc, final QName serviceName)
            throws WSDLException, TransformerException {
        String docString = XMLUtil.createStringFromDOMDocument(serviceDesc);
        Document tmpDoc = XMLUtil.createDocumentFromString(docString);
        Definition serviceDef = createDefinitionFromDocument(tmpDoc);
        return getServiceAddressFromWSDLDefinition(serviceDef, serviceName);
    }

    /**
     * Returns the service address contained in a WSDL 1.0 file
     *
     * @param wsdlFile
     *            The WSDL file
     * @return The service address attribute or null if not found
     * @throws WSDLException
     * @throws TransformerException
     */
    public static String getServiceAddressFromWSDLFile(final File wsdlFile,
            final QName serviceName) throws WSDLException, TransformerException {
        Document doc = createDocumentFromWSDL(wsdlFile);
        return getServiceAddressFromWSDLDocument(doc, serviceName);
    }

    /**
     * Returns the service name contained in a WSDL 1.1 dom Document
     *
     * @param serviceDef
     *            The dom Document
     * @return The service name attribute or null if not found
     */
    @SuppressWarnings("unchecked")
    public static QName[] getServiceNameFromWSDLDefinition(Definition serviceDef) {
        QName[] result = null;
        if (serviceDef != null) {
            Map<String, Service> services = serviceDef.getServices();
            result = new QName[services.size()];
            int i = 0;
            for (Service service : services.values()) {
                result[i++] = service.getQName();
            }
        }
        return result;
    }

    /**
     * Returns the service name contained in a WSDL 1.1 dom Document
     *
     * @param serviceDef
     *            The dom Document
     * @return The service name attribute or null if not found
     * @throws WSDLException
     * @throws TransformerException
     */
    public static QName[] getServiceNameFromWSDLDocument(
            final Document serviceDesc) throws WSDLException,
            TransformerException {
        String docString = XMLUtil.createStringFromDOMDocument(serviceDesc);
        Document tmpDoc = XMLUtil.createDocumentFromString(docString);
        Definition serviceDef = createDefinitionFromDocument(tmpDoc);
        return getServiceNameFromWSDLDefinition(serviceDef);
    }

    /**
     * Returns the service name contained in a WSDL 1.0 file
     *
     * @param wsdlFile
     *            The WSDL file
     * @return The service name attribute or null if not found
     * @throws WSDLException
     * @throws TransformerException
     */
    public static QName[] getServiceNameFromWSDLFile(final File wsdlFile)
            throws WSDLException, TransformerException {
        Document doc = createDocumentFromWSDL(wsdlFile);
        return getServiceNameFromWSDLDocument(doc);
    }

    /**
     * Returns WSDL 1.0 document target namespace
     *
     * @param serviceDesc
     *            The service description document
     * @return The target namespace or null if not found
     */
    public static String getTargetNSFromWSDLDefinition(
            final Definition serviceDef) {
        return serviceDef.getTargetNamespace();
    }

    /**
     * Returns WSDL 1.0 document target namespace
     *
     * @param serviceDesc
     *            The service description document
     * @return The target namespace or null if not found
     * @throws WSDLException
     * @throws TransformerException
     */
    public static String getTargetNSFromWSDLDocument(final Document serviceDesc)
            throws WSDLException, TransformerException {
        String docString = XMLUtil.createStringFromDOMDocument(serviceDesc);
        Document tmpDoc = XMLUtil.createDocumentFromString(docString);
        Definition serviceDef = createDefinitionFromDocument(tmpDoc);
        return getTargetNSFromWSDLDefinition(serviceDef);
    }

    /**
     * Search the service description for an operation
     *
     * @param serviceDesc
     *            dom Document representation of the service description
     * @param opName
     *            Name of the operation to search
     * @return true if the operation has been found or else false
     */
    public static boolean hasOperationNamed(final Definition serviceDef,
            final String opName, final QName serviceName) {
        BindingOperation op = findOperationNamed(serviceDef, opName,
                serviceName);
        return op != null;
    }

    /**
     * Search the service description for an operation
     *
     * @param serviceDesc
     *            dom Document representation of the service description
     * @param opName
     *            Name of the operation to search
     * @return true if the operation has been found or else false
     * @throws WSDLException
     * @throws TransformerException
     */
    public static boolean hasOperationNamed(final Document serviceDesc,
            final String opName, final QName serviceName) throws WSDLException,
            TransformerException {
        String docString = XMLUtil.createStringFromDOMDocument(serviceDesc);
        Document tmpDoc = XMLUtil.createDocumentFromString(docString);
        Definition serviceDef = createDefinitionFromDocument(tmpDoc);
        return hasOperationNamed(serviceDef, opName, serviceName);
    }

    /**
     * Search the service description for an operation. If the operation is
     * found , checks if it has a child node named 'output'
     *
     * @param serviceDesc
     *            dom Document representation of the service description
     * @param opName
     *            Name of the operation to search
     * @return true if the operation has been found and has the output child
     *         node or else false
     */
    public static boolean isInOutOperation(final Definition serviceDef,
            final String opName, final QName serviceName) {
        boolean inOut = false;
        BindingOperation op = findOperationNamed(serviceDef, opName,
                serviceName);
        if (op != null) {
            inOut = op.getBindingOutput() != null;
        }
        return inOut;
    }

    /**
     * Search the service description for an operation. If the operation is
     * found , checks if it has a child node named 'output'
     *
     * @param serviceDesc
     *            dom Document representation of the service description
     * @param opName
     *            Name of the operation to search
     * @return true if the operation has been found and has the output child
     *         node or else false
     * @throws WSDLException
     * @throws TransformerException
     */
    public static boolean isInOutOperation(final Document serviceDesc,
            final String opName, final QName serviceName) throws WSDLException,
            TransformerException {
        String docString = XMLUtil.createStringFromDOMDocument(serviceDesc);
        Document tmpDoc = XMLUtil.createDocumentFromString(docString);
        Definition serviceDef = createDefinitionFromDocument(tmpDoc);
        return isInOutOperation(serviceDef, opName, serviceName);
    }

    /**
     * Create a {@link Document} from a {@link Definition}
     *
     * @param def
     * @return
     * @throws WSDLException
     */
    protected static Document createDocumentFromDefinition(final Definition def)
            throws WSDLException {
        Document result = null;
        WSDLWriter writer = new WSDLWriterImpl();
        OutputStream outStream = new ByteArrayOutputStream();
        writer.writeWSDL(def, outStream);
        String documentToString = outStream.toString();
        result = XMLUtil.createDocumentFromString(documentToString);
        return result;
    }

    /**
     *
     * @param serviceDef
     * @param opName
     * @param serviceName
     * @return
     */
    @SuppressWarnings("unchecked")
    protected static BindingOperation findOperationNamed(
            final Definition serviceDef, final String opName,
            final QName serviceName) {
        BindingOperation result = null;
        boolean found = false;
        if (serviceDef != null) {

            Service service = findService(serviceDef, serviceName);
            if (service != null) {
                Map<String, Port> ports = service.getPorts();
                for (Port port : ports.values()) {
                    List<BindingOperation> ops = port.getBinding()
                            .getBindingOperations();
                    for (BindingOperation op : ops) {
                        if (op.getName().equals(opName)) {
                            result = op;
                            found = true;
                            break;
                        }
                    }
                    if (found) {
                        break;
                    }
                }
            }
        }
        return result;
    }

    /**
     * Return the Service definition for this service. This method is comparable
     * to definition.getService(qname_service), but due to some variation in
     * QName structure, we perform manually a equals(qname_service, qname) to
     * retreive the service description
     *
     * @param definition
     *            wsdl definition, can be null
     * @param service
     *            service to retrieve its definition, can be null
     * @return null if definition or service are null, or service not found in
     *         description
     */
    public static Service findService(final Definition definition,
            final QName service) {
        Service result = null;

        if (definition != null && service != null) {
            for (Iterator iter = definition.getServices().keySet().iterator(); iter
                    .hasNext()
                    && result == null;) {
                QName element = (QName) iter.next();
                if (equals(element, service)) {
                    result = definition.getService(element);
                }

            }
        }
        return result;
    }

    /**
     * Perform a qname.toString equality
     *
     * @param a
     *            can be null
     * @param b
     *            can be null
     * @return a.toString equals b.toString, or a=b=null
     */
    public static boolean equals(final QName a, final QName b) {
        boolean result = true;

        if (a != null && b != null) {
            result = a.toString().equals(b.toString());
        } else {
            result = a == null && b == null;
        }
        return result;
    }

    /**
     * Create a light WSDL2.0 document from the given parameters. All parameters
     * are required, otherwise a NullPointerException will be thrown. Namespaces
     * are managed.
     *
     * @param interfaceName
     *            must be non null
     * @param serviceName
     *            must be non null
     * @param endpointName
     *            must be non null
     * @return a WSDL2.0 light document
     */
    public static Document createLightWSDL20(final QName interfaceName,
            final QName serviceName, final String endpointName) {

        StringBuffer s = new StringBuffer();

        // retrieve the namespace if defined
        String targetNamespace = interfaceName.getNamespaceURI();
        if (StringHelper.isNullOrEmpty(interfaceName.getNamespaceURI())) {
            targetNamespace = serviceName.getNamespaceURI();
        }

        // write definition xmlns block
        s.append("<?xml version='1.0' encoding='UTF-8'?>\n");
        s.append("  <description ");
        if (!StringHelper.isNullOrEmpty(targetNamespace)) {
            s.append(" targetNamespace='" + targetNamespace + "'");
            s.append(" xmlns:tns='" + targetNamespace + "'");
        }
        s.append(" xmlns='http://www.w3.org/2006/01/wsdl'>\n");

        // write interface block
        s.append("    <interface name='" + interfaceName.getLocalPart()
                + "'></interface>\n");

        // write binding block
        s.append("    <binding name='" + endpointName + "'></binding>\n");

        // write service block (with interface and endpoint part)
        s.append("    <service name='" + serviceName.getLocalPart() + "'\n");
        if (!StringHelper.isNullOrEmpty(targetNamespace)) {
            s.append("             interface='tns:"
                    + interfaceName.getLocalPart() + "'>\n");
            s.append("      <endpoint binding='tns:" + endpointName
                    + "'></endpoint>\n");
        } else {
            s.append("             interface='" + interfaceName.getLocalPart()
                    + "'>\n");
            s.append("      <endpoint binding='" + endpointName
                    + "'></endpoint>\n");
        }

        s.append("    </service>\n");
        s.append("  </description>");

        // construct the document
        ByteArrayInputStream is = new ByteArrayInputStream(s.toString()
                .getBytes());

        Document doc = null;
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        try {
            DocumentBuilder builder = factory.newDocumentBuilder();
            doc = builder.parse(is);
        } catch (Exception e) {
            // this kind of exception should never be thrown...
        }
        return doc;
    }

    /**
     * Change all the addresses prefix of the WSDL document to the new one.
     *
     * @param doc
     * @param newPrefix
     * @return
     */
    @SuppressWarnings("unchecked")
    public static Document changeAdressPrefixInWSDL(Document doc,
            String newPrefix) throws Exception {

        WSDLFactory wsdlFactory = WSDLFactory.newInstance();
        WSDLReader reader = wsdlFactory.newWSDLReader();
        Definition def = reader.readWSDL(null, doc);

        Map services = def.getAllServices();
        Iterator iter = services.keySet().iterator();
        while (iter.hasNext()) {
            Service service = (Service) services.get(iter.next());

            Map<String, Port> ports = service.getPorts();
            for (Port port : ports.values()) {
                List<ExtensibilityElement> extensibilityElements = port
                        .getExtensibilityElements();

                for (ExtensibilityElement element : extensibilityElements) {
                    if (element instanceof SOAPAddress) {
                        SOAPAddress address = (SOAPAddress) element;
                        address.setLocationURI(newPrefix + "/"
                                + service.getQName().getLocalPart());

                    } else if (element instanceof SOAP12Address) {
                        SOAP12Address address = (SOAP12Address) element;
                        address.setLocationURI(newPrefix + "/"
                                + service.getQName().getLocalPart());

                    } else if (element instanceof HTTPAddress) {
                        HTTPAddress address = (HTTPAddress) element;
                        address.setLocationURI(newPrefix + "/"
                                + service.getQName().getLocalPart());
                    }
                }
            }
        }
        return createDocumentFromDefinition(def);
    }

    /**
     * Replace the wsdl ports address with the given one
     *
     * @param doc
     * @param localAddress
     * @return
     * @throws TransformerException
     */
    public static Document replaceAddressPrefix(Document doc,
            String localAddress) throws TransformerException {

        // get services
        NodeList services = doc.getElementsByTagName("wsdl:service");
        for (int i = 0; i < services.getLength(); i++) {
            Element serviceElement = (Element) services.item(i);
            String serviceName = serviceElement.getAttribute("name");
            NodeList ports = serviceElement.getElementsByTagName("wsdl:port");

            // get the ports
            for (int j = 0; j < ports.getLength(); j++) {
                Element portElement = (Element) ports.item(j);

                // get addresses
                NodeList addresses = portElement
                        .getElementsByTagName("soap:address");

                for (int k = 0; k < addresses.getLength(); k++) {
                    Element addr = (Element) addresses.item(k);
                    addr.setAttribute("location", localAddress + "/"
                            + serviceName);
                }
                addresses = portElement.getElementsByTagName("soap12:address");
                for (int k = 0; k < addresses.getLength(); k++) {
                    Element addr = (Element) addresses.item(k);
                    addr.setAttribute("location", localAddress + "/"
                            + serviceName);
                }
                addresses = portElement.getElementsByTagName("http:address");
                for (int k = 0; k < addresses.getLength(); k++) {
                    Element addr = (Element) addresses.item(k);
                    addr.setAttribute("location", localAddress + "/"
                            + serviceName);
                }
            }
        }
        String docString = XMLUtil.createStringFromDOMDocument(doc);
        return XMLUtil.createDocumentFromString(docString);
    }

    /**
     * Replace all the addresses by the given one. Ignore all the name spaces.
     *
     * @param doc
     * @param localAddress
     * @return
     * @throws TransformerException
     */
    public static Document replaceAddressPrefixIgnoreNS(Document doc,
            String localAddress) throws TransformerException {
        List<Node> services = WSDLHelper.getChildNodes(doc.getFirstChild(),
                "service");
        for (Node service : services) {
            String serviceName = null;
            if (service instanceof Element) {
                Element e = (Element) service;
                serviceName = e.getAttribute("name");
            }

            List<Node> ports = WSDLHelper.getChildNodes(service, "port");

            for (Node port : ports) {
                List<Node> addresses = WSDLHelper
                        .getChildNodes(port, "address");
                for (Node address : addresses) {
                    if (address instanceof Element) {
                        Element a = (Element) address;
                        a.setAttribute("location", localAddress + "/"
                                + serviceName);
                    }
                }
            }
        }
        String docString = XMLUtil.createStringFromDOMDocument(doc);
        return XMLUtil.createDocumentFromString(docString);
    }

    /**
     * Replace the service address in the wsdl:address element, not in the
     * ports.
     *
     * @param doc
     * @param localAddress
     * @param newServiceName
     * @return
     * @throws TransformerException
     */
    public static Document replaceAddressServiceIgnoreNS(Document doc,
            String localAddress, String newServiceName)
            throws TransformerException {
        List<Node> services = WSDLHelper.getChildNodes(doc.getFirstChild(),
                "service");
        for (Node service : services) {
            List<Node> ports = WSDLHelper.getChildNodes(service, "port");

            for (Node port : ports) {
                List<Node> addresses = WSDLHelper
                        .getChildNodes(port, "address");
                for (Node address : addresses) {
                    if (address instanceof Element) {
                        Element a = (Element) address;
                        a.setAttribute("location", localAddress + "/"
                                + newServiceName);
                    }
                }
            }
        }
        String docString = XMLUtil.createStringFromDOMDocument(doc);
        return XMLUtil.createDocumentFromString(docString);
    }

    /**
     * Replace the address
     *
     * @param doc
     * @param localAddress
     * @return
     * @throws TransformerException
     */
    public static Document replaceAddressAndServiceIgnoreNS(Document doc,
            String newAddress, String newServiceName)
            throws TransformerException {
        List<Node> services = WSDLHelper.getChildNodes(doc.getFirstChild(),
                "service");
        for (Node service : services) {
            if (service instanceof Element) {
                Element e = (Element) service;
                if (e.getAttribute("name") != null)
                    try {
                        e.setAttribute("name", newServiceName);
                    } catch (DOMException e1) {
                    }
            }

            List<Node> ports = WSDLHelper.getChildNodes(service, "port");

            for (Node port : ports) {
                List<Node> addresses = WSDLHelper
                        .getChildNodes(port, "address");
                for (Node address : addresses) {
                    if (address instanceof Element) {
                        Element a = (Element) address;
                        a.setAttribute("location", newAddress + "/"
                                + newServiceName);
                    }
                }
            }
        }
        String docString = XMLUtil.createStringFromDOMDocument(doc);
        return XMLUtil.createDocumentFromString(docString);
    }

    /**
     * Get the children of the current with name ending with the given one
     * (ignore NS)
     *
     * @param element
     * @param name
     * @return
     */
    public static List<Node> getChildNodes(Node element, String name) {
        List<Node> elements = new ArrayList<Node>();
        NodeList children = element.getChildNodes();
        for (int i = 0; i < children.getLength(); i++) {
            Node tmp = children.item(i);
            if (tmp.getNodeName().endsWith(name)) {
                elements.add(tmp);
            }
        }
        return elements;
    }

    /**
     * Print the WSDL to the output stream
     *
     * @param doc
     * @param out
     */
    public static void printWSDL(Document doc, OutputStream out) {
        try {
            String buffer = XMLUtil.createStringFromDOMDocument(doc);
            out.write(buffer.getBytes());
        } catch (IOException e) {
            e.printStackTrace();
        } catch (TransformerException e) {
            e.printStackTrace();
        }

    }
}
